#ifndef _ASMARM_SUSPEND_H
#define _ASMARM_SUSPEND_H
/*
 * Based on code
 * Copyright 2005  Sony Corporation
 * Copyright 2003-2004 Nigel Cunningham <ncunningham@linuxmail.org>
 * Copyright 2001-2002 Pavel Machek <pavel@suse.cz>
 * Copyright 2001 Patrick Mochel <mochel@osdl.org>
 */

/* image of the saved processor state */
struct suspend2_saved_context {
	/* general registers */
	__u32 r[15];

	/* coprocessor 15 registers */
/*	__u32 ID_code;    read only reg */
/*	__u32 cache_type; read only reg */
/*	__u32 TCM_stat;   read only reg */
	__u32 CR;
	__u32 TTBR;
	__u32 DACR;
	__u32 D_FSR;
	__u32 I_FSR;
	__u32 FAR;
/*	__u32 COR;        write only reg */
/*	__u32 TLBOR;      write only reg */
	__u32 D_CLR;
	__u32 I_CLR;
	__u32 D_TCMRR;
	__u32 I_TCMRR;
	__u32 TLBLR;
	__u32 FCSE;
	__u32 CID;
} __attribute__((packed));
typedef struct suspend2_saved_context suspend2_saved_context_t;

/* temporary storage */
extern struct suspend2_saved_context suspend2_saved_context;

static inline void suspend2_arch_save_processor_context(void)
{
	/* save general registers */
	asm volatile ("stmia %0, {r4-r14}"
		      :: "r" (suspend2_saved_context.r));
	/* save coprocessor 15 registers */
	asm volatile ("mrc p15, 0, %0, c1, c0, 0"
		      : "=r" (suspend2_saved_context.CR));
	asm volatile ("mrc p15, 0, %0, c3, c0, 0"
		      : "=r" (suspend2_saved_context.DACR));
	asm volatile ("mrc p15, 0, %0, c5, c0, 0"
		      : "=r" (suspend2_saved_context.D_FSR));
	asm volatile ("mrc p15, 0, %0, c5, c0, 1"
		      : "=r" (suspend2_saved_context.I_FSR));
	asm volatile ("mrc p15, 0, %0, c6, c0, 0"
		      : "=r" (suspend2_saved_context.FAR));
	asm volatile ("mrc p15, 0, %0, c9, c0, 0"
		      : "=r" (suspend2_saved_context.D_CLR));
	asm volatile ("mrc p15, 0, %0, c9, c0, 1"
		      : "=r" (suspend2_saved_context.I_CLR));
	asm volatile ("mrc p15, 0, %0, c9, c1, 0"
		      : "=r" (suspend2_saved_context.D_TCMRR));
	asm volatile ("mrc p15, 0, %0, c9, c1, 1"
		      : "=r" (suspend2_saved_context.I_TCMRR));
	asm volatile ("mrc p15, 0, %0, c10, c0, 0"
		      : "=r" (suspend2_saved_context.TLBLR));
	asm volatile ("mrc p15, 0, %0, c13, c0, 0"
		      : "=r" (suspend2_saved_context.FCSE));
	asm volatile ("mrc p15, 0, %0, c13, c0, 1"
		      : "=r" (suspend2_saved_context.CID));
	asm volatile ("mrc p15, 0, %0, c2, c0, 0"
		      : "=r" (suspend2_saved_context.TTBR));
}

static inline void suspend2_arch_restore_processor_context(void)
{
	/* restore coprocessor 15 registers */
	asm volatile ("mcr p15, 0, %0, c2, c0, 0"
		      :: "r" (suspend2_saved_context.TTBR));
	asm volatile ("mcr p15, 0, %0, c13, c0, 1"
		      :: "r" (suspend2_saved_context.CID));
	asm volatile ("mcr p15, 0, %0, c13, c0, 0"
		      :: "r" (suspend2_saved_context.FCSE));
	asm volatile ("mcr p15, 0, %0, c10, c0, 0"
		      :: "r" (suspend2_saved_context.TLBLR));
	asm volatile ("mcr p15, 0, %0, c9, c1, 1"
		      :: "r" (suspend2_saved_context.I_TCMRR));
	asm volatile ("mcr p15, 0, %0, c9, c1, 0"
		      :: "r" (suspend2_saved_context.D_TCMRR));
	asm volatile ("mcr p15, 0, %0, c9, c0, 1"
		      :: "r" (suspend2_saved_context.I_CLR));
	asm volatile ("mcr p15, 0, %0, c9, c0, 0"
		      :: "r" (suspend2_saved_context.D_CLR));
	asm volatile ("mcr p15, 0, %0, c6, c0, 0"
		      :: "r" (suspend2_saved_context.FAR));
	asm volatile ("mcr p15, 0, %0, c5, c0, 1"
		      :: "r" (suspend2_saved_context.I_FSR));
	asm volatile ("mcr p15, 0, %0, c5, c0, 0"
		      :: "r" (suspend2_saved_context.D_FSR));
	asm volatile ("mcr p15, 0, %0, c3, c0, 0"
		      :: "r" (suspend2_saved_context.DACR));
	asm volatile ("mcr p15, 0, %0, c1, c0, 0"
		      :: "r" (suspend2_saved_context.CR));

	/* restore general registers */
	asm volatile ("ldmia r3, {r4-r14}" : "=m" (suspend2_saved_context.r));
}

static inline void save_context(void)
{
}

static inline void restore_context(void)
{
}

static inline void suspend2_arch_pre_copy(void)
{
}

static inline void suspend2_arch_post_copy(void)
{
}

static inline void suspend2_arch_pre_copyback(void)
{
}

static inline void suspend2_arch_post_copyback(void)
{
}

static inline void suspend2_arch_flush_caches(void)
{
}
#endif
